﻿// CleanPages.jsx
// An InDesign JavaScript
//
// Cleaning the pages of active document
//
// Copyright: Sho Nuff - shonuff@ya.ru
// License: MIT
//
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to 
// deal in the Software without restriction, including without limitation the 
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or 
// sell copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in 
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
// IN THE SOFTWARE.
//
//
// 2012-04-25

#target indesign

if (!app.documents.length) exit(); // нет открытых документов

// все глобальные переменные
var 
	aDoc =      app.activeDocument,
	// script preferences
	prefsFile = new File(Folder.userData + '/CleanPages.conf' );
	prefs =     {
		startPageNumber:  0,
		endPageNumber:    aDoc.pages.length - 1,
		activeLayers:     '',
		removeTextFrames: true,
		removePolygons:   true,
		removeGroups:     true,
		removeOther:      true,
		removeLocked:     false
	};

// прочитаем предыдущие настройки
if (prefsFile.exists && prefsFile.open('r')) {
	
	try {
		prefs = eval(prefsFile.read());
	} catch (e) {}
	
	prefsFile.close();
}

// уточним настройки
if (prefs.startPageNumber > aDoc.pages.length - 1) prefs.startPageNumber = aDoc.pages.length - 1;
if (prefs.endPageNumber > aDoc.pages.length - 1) prefs.endPageNumber = aDoc.pages.length - 1;

// создаем диалог
(function() {
	var 
		pageNames =  [],
		layerNames = [];
	
	// собираем в массив имена страниц
	// если нужно дбавляем префикс
	for (var i = 0, l = aDoc.pages.length; i < l; i++) {
		var 
			page =    aDoc.pages[i],
			section = page.appliedSection;
		
		pageNames.push(
			(section.includeSectionPrefix ? '' :  section.sectionPrefix) +
			page.name);
	}

	// собираем в массив названия слоев
	for (var i = 0, l = aDoc.layers.length; i < l; i++) {
		layerNames.push(aDoc.layers[i].name);
	}
	var dialog = app.dialogs.add({name: 'CleanPages'});
	var colMain = dialog.dialogColumns.add();

	// выбор первой страницы диапазона
	var rowStartPage = colMain.dialogRows.add();
	rowStartPage.staticTexts.add({staticLabel: localize({en: 'Start Page', ru: 'Начать со страницы:'}), minWidth: 150});
	var ddStartPage = rowStartPage.dropdowns.add({stringList: pageNames, selectedIndex: prefs.startPageNumber, minWidth: 100});

	// выбор последней страницы диапазона
	var rowEndPage = colMain.dialogRows.add();
	rowEndPage.staticTexts.add({staticLabel: localize({en: 'End Page:', ru: 'Закончить страницей:'}), minWidth: 150});
	var ddEndPage = rowEndPage.dropdowns.add({stringList: pageNames, selectedIndex: prefs.endPageNumber, minWidth: 100});

	// список слоев
	var colLayer = colMain.dialogRows.add().borderPanels.add().dialogColumns.add();
	colLayer.dialogRows.add().staticTexts.add({staticLabel: localize({
			en: 'Clean Layers:                                   ', 
			ru: 'Очистить слои:                                  '})});
	var cbLayers = [];

	for (var i = 0, l = layerNames.length; i < l; i++) {
		var layerName = layerNames[i];
		
		cbLayers[i] = colLayer.dialogRows.add().checkboxControls.add({
			staticLabel: layerName, 
			checkedState: prefs.activeLayers.indexOf('\t' + layerName + '\t') >= 0
		});
	}
	
	// список объектов
	var colObjects = colMain.dialogRows.add().borderPanels.add().dialogColumns.add();
	colObjects.dialogRows.add().staticTexts.add({staticLabel: localize({
			en:'Remove:                                        ',
			ru: 'Удалять объекты:                              '})});
	
	var cbTextFrames = colObjects.dialogRows.add().checkboxControls.add({
		staticLabel: localize({en: 'Text Frames', ru: 'Текстовые фреймы'}), 
		checkedState: !!prefs.removeTextFrames
	});
	var cbPolygons = colObjects.dialogRows.add().checkboxControls.add({
		staticLabel: localize({en: 'Lines, Ovals, Polygons', ru: 'Линии, овалы, многоугольники'}), 
		checkedState: !!prefs.removePolygons
	});
	var cbGroups = colObjects.dialogRows.add().checkboxControls.add({
		staticLabel: localize({en:'Groups', ru: 'Группы'}), 
		checkedState: !!prefs.removeGroups
	});
	var cbOther = colObjects.dialogRows.add().checkboxControls.add({
		staticLabel: localize({en:'Other', ru: 'Прочие'}), 
		checkedState: !!prefs.removeOther
	});

	// очистка залоченных
	var cbLocked = colMain.dialogRows.add().checkboxControls.add({
		staticLabel: localize({en: 'Remove Locked', ru: 'Очищать заблокированные'}), 
		checkedState: !!prefs.removeLocked
	});
	
	while (true) {
		// покажем диалог
		if (!dialog.show()) {
			// если нажата кнопка отмены - выход
			dialog.destroy(); 
			exit();
		}
	
		// обновим prefs
		prefs.startPageNumber = Math.min(ddStartPage.selectedIndex, ddEndPage.selectedIndex);
		prefs.endPageNumber =   Math.max(ddStartPage.selectedIndex, ddEndPage.selectedIndex);
		var str = '\t';
		
		for (var i = cbLayers.length; i--; ) {
			if (!cbLayers[i].checkedState) continue;
			str += layerNames[i] + '\t';
		}
		if (str.length < 2) {
			alert(localize({en: 'Select Layer, please', ru: 'Нужно указать хотя бы один слой'}));
			continue;
		}
		prefs.activeLayers = str;
		
		if (!(cbTextFrames.checkedState || 
			cbPolygons.checkedState || cbOther.checkedState)) {
			
			alert(localize({en: 'Choice Type of Objects', ru: 'Не выбраны объекты для удаления'}));
			continue;
		}
		prefs.removeTextFrames = cbTextFrames.checkedState;
		prefs.removePolygons =   cbPolygons.checkedState;
		prefs.removeGroups =     cbGroups.checkedState;
		prefs.removeOther =      cbOther.checkedState;
		prefs.removeLocked =     cbLocked.checkedState;
		
		break;
	}
	dialog.destroy();
})();
// конец диалога

// сохраним prefs
if (!prefsFile.open('w') || !prefsFile.write(prefs.toSource())) {
	alert(localise({en: 'Error. Can not save settings', ru: 'Ошибка\nНе удалось сохранить настройки.'}));
}
prefsFile.close();

// обходит коллекцию pageItems и удаляет подходящие по параметрам элементы
function CleanPageItems(pageItems) {
	seekPageItem:
	for (var i = pageItems.length; i--; ) {
		var pageItem = pageItems[i];
		
		// пропустим залоченные
		if (pageItem.locked && !prefs.removeLocked) continue;

		// проверим слой
		if (pageItem.itemLayer.locked) continue;
		if (prefs.activeLayers.indexOf('\t' + pageItem.itemLayer.name + '\t') < 0) continue;
		
		// удаляем только объекты размещенные прямо на странице
		if (pageItem.parent.constructor.name != 'Page') continue;
		
		// проверим конструктор
		switch (pageItem.constructor.name) {
			case 'TextFrame':
				if (!prefs.removeTextFrames) {
					continue seekPageItem;
				}
				break;
			
			case 'GraphicLine':
			case 'Oval':
			case 'Polygon':
			case 'Rectangle':
				if (!prefs.removePolygons) continue seekPageItem;
				break;
			
			case 'Group':
				if (!prefs.removeGroups) continue seekPageItem;
				break;
			
			default: 
				if (!prefs.removeOther) continue seekPageItem;
		}
		pageItem.locked = false;
		pageItem.remove();
	}
}

// обернем основные операции в одну функцию 
// чтобы все действия скрипта можно было отменить одной командой
function CleanPages() {
	var unlockedLayers = [];
		
	// разлочим все слои
	if (prefs.removeLocked) {
		for (var i = aDoc.layers.length; i--; ) {
			var layer = aDoc.layers[i];
			
			if (layer.locked) {
				unlockedLayers.push(layer);
				layer.locked = false;
			}
		}
	}

	// обход страниц
	for (var i = prefs.startPageNumber; i <= prefs.endPageNumber; i++) {
		CleanPageItems(aDoc.pages[i].allPageItems);
	}

	// залочим обратно слои
	for (var i = unlockedLayers.length; i--; ) {
		unlockedLayers[i].locked = true;
	}
}

if (parseFloat(app.version.split('.')[0]) < 6) {
	// в старых версиях ID просто вызовем функцию
	CleanPages();
} else {
	// в ID 6 и старше cгруппируем все операции в одну
	app.doScript(CleanPages, undefined, undefined, UndoModes.FAST_ENTIRE_SCRIPT, 'Clean Pages');
}
